@@ -81,7 +81,9 @@ class TestBaseEnvironment:
         assert env.action_shape == np.array(action).shape
         data = env.step_with_dt(action, dt=dt)
         assert isinstance(data, tuple)
-        assert env.obs_shape == obs.shape
+        assert (
+            env.obs_shape == obs.shape
+        ), f"env.obs_shape {tuple(env.obs_shape)}, obs: {obs.shape}"
 
     @pytest.mark.parametrize("dt", [1, 3])
     def test_step(self, env, dt):
@@ -171,6 +173,8 @@ class TestGymEnvironment(TestBaseEnvironment):
         assert hasattr(env, "action_space")
         # Singleton / delayed envs may not be properly initialized. In that case test separately
         if env.sample_action() is not None:
+            if env.action_space is None:
+                env.setup()
             action = env.action_space.sample()
             assert action is not None
 
