@@ -1,11 +1,11 @@
 """Implement the ``plangym`` API for ``dm_control`` environments."""
-from typing import Iterable, Union
+from typing import Iterable
 import warnings
 
 from gym.spaces import Box, Space
 import numpy as np
 
-from plangym.core import PlanEnvironment, wrap_callable
+from plangym.core import PlangymEnv, wrap_callable
 
 
 try:
@@ -16,7 +16,7 @@ except Exception:
     novideo_mode = True
 
 
-class DMControlEnv(PlanEnvironment):
+class DMControlEnv(PlangymEnv):
     """
     Wrap the dm_control library, so it can work for planning problems.
 
@@ -58,12 +58,8 @@ class DMControlEnv(PlanEnvironment):
             render_mode: None|human|rgb_array
         """
         self._visualize_reward = visualize_reward
-        self._render_i = 0
         self.viewer = []
-        self._last_time_step = None
         self._viewer = None
-        self._observation_space = None
-        self._action_space = None
         name, self._domain_name, self._task_name = self._parse_names(name, domain_name, task_name)
         super(DMControlEnv, self).__init__(
             name=name,
@@ -83,18 +79,13 @@ class DMControlEnv(PlanEnvironment):
     @property
     def observation_space(self) -> Space:
         """Return the observation_space of the environment."""
-        return self._observation_space
+        return self._obs_space
 
     @property
     def physics(self):
         """Alias for gym_env.physics."""
         return self.gym_env.physics
 
-    @property
-    def render_mode(self) -> str:
-        """Return how the game will be rendered. Values: None | human | rgb_array."""
-        return self._render_mode
-
     @property
     def domain_name(self) -> str:
         """Return the name of the agent in the current simulation."""
@@ -120,10 +111,6 @@ class DMControlEnv(PlanEnvironment):
         name = "-".join([domain_name, task_name])
         return name, domain_name, task_name
 
-    def sample_action(self) -> Union[int, np.ndarray]:
-        """Return a valid action that can be used to step the Environment."""
-        return self.action_space.sample()
-
     def init_gym_env(self):
         """Initialize the environment instance that the current class is wrapping."""
         from dm_control import suite
@@ -133,9 +120,7 @@ class DMControlEnv(PlanEnvironment):
             task_name=self.task_name,
             visualize_reward=self._visualize_reward,
         )
-        self._render_i = 0
         self.viewer = []
-        self._last_time_step = None
         self._viewer = None if novideo_mode else rendering.SimpleImageViewer()
         return env
 
@@ -143,30 +128,22 @@ class DMControlEnv(PlanEnvironment):
         """Initialize the target :class:`gym.Env` instance."""
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            self._gym_env = self.init_gym_env()
-            if self._wrappers is not None:
-                self.apply_wrappers(self._wrappers)
-            self._action_space = Box(
-                low=self.action_spec().minimum,
-                high=self.action_spec().maximum,
-                dtype=np.float32,
-            )
-            shape = self.reset(return_state=False).shape
-            self._observation_space = Box(low=-np.inf, high=np.inf, shape=shape, dtype=np.float32)
+            super(DMControlEnv, self).setup()
+
+    def init_spaces(self):
+        """Initialize the action_space and the observation_space of the environment."""
+        self._action_space = Box(
+            low=self.action_spec().minimum,
+            high=self.action_spec().maximum,
+            dtype=np.float32,
+        )
+        shape = self.reset(return_state=False).shape
+        self._obs_space = Box(low=-np.inf, high=np.inf, shape=shape, dtype=np.float32)
 
     def action_spec(self):
         """Alias for the environment's ``action_spec``."""
         return self.gym_env.action_spec()
 
-    def seed(self, seed=None):
-        """Seed the underlying :class:`gym.Env`."""
-        np.random.seed(seed)
-        # self.gym_env.seed(seed)
-
-    def get_image(self) -> np.ndarray:
-        """Return and RGB array representing the current state of the environment."""
-        return self.render(mode="rgb_array")
-
     def render(self, mode="human"):
         """
         Store all the RGB images rendered to be shown when the `show_game`\
@@ -195,26 +172,18 @@ class DMControlEnv(PlanEnvironment):
             self._viewer.imshow(img)
             time.sleep(sleep)
 
-    def reset(self, return_state: bool = True) -> [np.ndarray, tuple]:
+    def process_obs(self, obs, **kwargs) -> np.ndarray:
         """
-        Reset the environment and returns the first observation, or the first\
-         (state, obs) tuple.
+        Get the environment observation from a time_step object.
 
         Args:
-            return_state: If true return a also the initial state of the env.
+            obs: Time step object returned after stepping the environment.
+            **kwargs: Ignored
 
         Returns:
-            Observation of the environment if `return_state` is False. Otherwise,
-            return (state, obs) after reset.
+            Numpy array containing the environment observation.
         """
-        time_step = self.gym_env.reset()
-        # observed, *_ = self.step_with_dt(action=self.sample_action(), dt=20)
-        observed = self._time_step_to_obs(time_step)
-        self._render_i = 0
-        if not return_state:
-            return observed
-        else:
-            return self.get_state(), observed
+        return self._time_step_to_obs(time_step=obs)
 
     def set_state(self, state: np.ndarray) -> None:
         """
@@ -247,10 +216,9 @@ class DMControlEnv(PlanEnvironment):
         """Transform the returned time_step object to a compatible gym tuple."""
         info = {}
         time_step = self.gym_env.step(action)
-        obs = self._time_step_to_obs(time_step)  # TODO: Optimize this so we only do it once
-        env_end = time_step.last()
+        obs = time_step
+        terminal = time_step.last()
         _reward = time_step.reward if time_step.reward is not None else 0.0
-        terminal = self.terminal_condition(obs, _reward, env_end, info, action)
         reward = _reward + self._reward_step
         return obs, reward, terminal, info
 
