@@ -530,57 +530,3 @@ class Montezuma(AtariEnvironment):
             int(cur_lives),
         )
         self.gym_env.restore(data)
-
-    def step(self, action: np.ndarray, state: np.ndarray = None, dt: int = 1) -> tuple:
-        """
-        Take dt simulation steps and make the environment evolve in multiples of frameskip.
-
-        The info dictionary will contain a boolean called 'lost_live' that will
-        be true if a life was lost during the current step.
-
-        Args:
-            action: Chosen action applied to the environment.
-            state: Set the environment to the given state before stepping it.
-            dt: Consecutive number of times that the action will be applied.
-
-        Returns:
-            if states is None returns (observs, rewards, ends, infos)
-            else returns(new_states, observs, rewards, ends, infos)
-        """
-        if state is not None:
-            self.set_state(state)
-        reward = 0
-        _end, lost_live = False, False
-        info = {"lives": -1}
-        terminal = False
-        game_end = False
-        for _ in range(int(dt)):
-            for _ in range(self.frameskip):
-                obs, _reward, _end, _info = self.gym_env.step(action)
-                _info["lives"] = _info.get("ale.lives", -1)
-                lost_live = info["lives"] > _info["lives"] or lost_live
-                game_end = game_end or _end
-                terminal = terminal or game_end
-                terminal = terminal or lost_live if self.episodic_life else terminal
-                info = _info.copy()
-                reward += _reward
-                if _end:
-                    break
-            if _end:
-                break
-        # This allows to get the original values even when using an episodic life environment
-        info["terminal"] = terminal
-        info["lost_live"] = lost_live
-        info["game_end"] = game_end
-        if state is not None:
-            new_state = self.get_state()
-            data = new_state, obs, reward, terminal, info
-        else:
-            data = obs, reward, terminal, info
-        if _end and self.autoreset:
-            self.gym_env.reset()
-        return data
-
-    def render(self, *args, **kwargs):
-        """Render the environment using OpenGL. This wraps the OpenAI render method."""
-        return self.gym_env.render(*args, **kwargs)
