@@ -1,8 +1,8 @@
 from gym.wrappers import TimeLimit
 import pytest
 
+from plangym.api_tests import batch_size, display, TestBaseEnvironment, TestGymEnvironment
 from plangym.classic_control import ClassicControl
-from tests.api_tests import batch_size, display, TestBaseEnvironment, TestGymEnvironment
 
 
 environments = [
@@ -18,4 +18,4 @@ environments = [
 def env(request) -> ClassicControl:
     name = request.param
     timelimit = [(TimeLimit, {"max_episode_steps": 1000})]
-    return ClassicControl(name=name, delay_init=name == "CartPole-v0", wrappers=timelimit)
+    return ClassicControl(name=name, delay_setup=name == "CartPole-v0", wrappers=timelimit)
